# Learning Programming with TADS 3

This repository showcases learning programming with the TADS 3 development system. The classes I'll be teaching this with are largely from grades fifth through seventh. The focus is on getting a good grounding in programming concepts while also learning a bit about game development. Because TADS 3 is focused on text adventure style games (also called "interactive fiction") there will be no emphasis on aspects like graphical or sound design.

## Testing

### tUnit.t

This is based on code provided by Marco Bizzarri in a project he referred to as "TADS3 Unit Test Sample". I've expanded upon this original idea, only slightly.

The idea here is that you can create a test project that execute a series tests backed up by assertions or expectations. Your code an assert or expect some condition to be the case.

You can see this in action by looking at the `makefile_test.t3m` file. This will compile a test project and create that as `goldskull_test.t3`. However you never need to run that game image. Instead what happens is that during the compilation process a series of tests are run. Those tests can be found in the `tests` folder.

### tSpec.t and tSpec.h

The original extension was called `tests.t` and apparently written by Jim Aikin, Ben Cressey and Eric Eve. There was a `tests2.t` extension that expanded upon that original a bit, written by NR Turner. I've made a few slight modifications.

You can define `Test` objects like this:

```python
playerInfo : Test
  testName = 'player info'
  location = outsideCave
  testList = [ 'x me', 'i' ]
;

basicSenses : Test
  testName = 'basic senses'
  testList = [ 'look', 'listen', 'smell' ]
;
```

The `test` command can be used with any `Test` object defined in the source code. For example, with the above tests in place, the following commands could be given:

    test player info
    test basic senses
    test "player info"
    test "basic senses"

You simply pass in the name you gave to the test. The quotes are optional.

Note that in the first test defined above, the player character will be moved to `outsideCave` before the test. In the second test, the actions will be attempted wherever the player character is when the test is run.

Here's another example:

```python
readWithLightSource : Test
	testName = 'read with light source'
	testHolding = [ matchbook, slate ]
	testList = [ 'light match', 'read slate' ]
;
```

Here the items `matchbook` and `slate` will be moved into the player character's inventory before the test.

Unless you're planning to refer to the `Test` object in some other part of your code, you can save a bit of typing by making it an anonymous object.

Alternatively, you can use the templates defined in tSpec.h. With that template structure, you can create your test objects more concisely:

```python
Test 'cave movement' ['in', 'out', 'enter', 'exit' ] 
Test 'north to cave' ['north'] @outsideCave;
Test 'in to cave' ['in'] @outsideCave;
Test 'enter cave' ['enter'] @outsideCave;
Test 'get skull' ['put rock on pedestal', 'take skull'] [smallRock]
Test 'get skull' ['put rock on pedestal', 'take skull'] [smallRock] @insideCave
```

Regarding the templates, there is an optional header file called `tSpec.h` and its purpose is to define templates for the `Test` class that's defined in `tSpec.t`. Once you have added `tSpec.t` to your project, you can include this header file:

```python
include "tSpec.h"
```

You should do this in each of your `.t` files to use these templates.

You can also make a test suite of sorts:

```python
testCaveEntering : Test
  testName = 'nav to cave'
  testList =
  [
    'test north to cave',
    'test in to cave',
    'test enter cave'
  ]
;
```

The commands `list tests` and `list tests fully` can be used to list your test scripts from within the running game.
