#include <adv3.h>
#include <en_us.h>

#include <file.h>
#include <lookup.h>

class Test : object
	testName = 'nil'
	testList = ['z']
	location = nil
	testHolding = []

	// testFile = 'TEST_' + testName + '.TCMD'
	testFile = static new WeakRefLookupTable(1,1)

	getFile() {
		local tempFile = testFile[0];

		if (!tempFile) {
			tempFile = new TemporaryFile();
			testFile[0] = tempFile;
		}
		
		return tempFile;
	}

	getHolding() {
		testHolding.forEach({x: x.baseMoveInto(gActor)});
	}

	run() {
		"Testing sequence: \"<<testName>>\". ";
		local testFile = getFile();
		local out = File.openTextFile(testFile, FileAccessWrite);

		testList.forEach({x: out.writeFile('><<x>>\n')});
		out.closeFile();

		if (location)
			gActor.moveIntoForTravel(location);

		getHolding();
		setScriptFile(testFile);
	}
;

#ifdef __DEBUG

DefineSystemAction(ListTests)
  fully = nil

  execSystemAction {
		if(allTests.lst.length == 0) {
      reportFailure('No test scripts are defined. ');
      exit;
    }

    foreach(local testObj in allTests.lst) {
      "<<testObj.testName>>";

      if(gAction.fully) {
        ": ";
        foreach(local txt in testObj.testList)
					"<<txt>>/";
        if (testObj.location)
          " (in <<testObj.location.roomName>>)";
        if (testObj.testHolding.length()) {
          " (holding ";
					objectLister.showSimpleList(testObj.testHolding);
					")";
        }
      }
      "\n";
		}
	}
;

VerbRule(ListTests)
  ('list' | 'l') 'tests' (| 'fully' -> fully)
  : ListTestsAction
  verbPhrase = 'list/listing test scripts'
;

DefineLiteralAction(Test)
  execAction() {
    local target = getLiteral().toLower();
    local script = allTests.valWhich({x: x.testName.toLower == target});

    if (script)
      script.run();
    else
      "Test sequence not found. ";
   }
;

VerbRule(Test)
  'test' singleLiteral
  : TestAction
  verbPhrase = 'test/testing (what)'
;

allTests : object
  lst() {
    if (lst_ == nil)
      initLst();
    return lst_;
  }

  initLst() {
    lst_ = new Vector(50);
    local obj = firstObj();

    while (obj != nil) {
      if(obj.ofKind(Test))
        lst_.append(obj);
      obj = nextObj(obj);
    }

    lst_ = lst_.toList();
  }

  valWhich(cond) {
    if (lst_ == nil)
      initLst();
    return lst_.valWhich(cond);
  }

  lst_ = nil
;

#endif
