#include <adv3.h>

class TestCase : object
  expect(condition) {
    if (!condition) {
      throw new ExpectationFailure('');
    }
  }

  assert(condition) {
    if (!condition) {
      throw new AssertionFailure('');
    }
  }

  assertTrue(condition) {
    if (!condition) {
      throw new AssertionFailure('');
    }
  }

  assertFalse(condition) {
    if (condition) {
      throw new AssertionFailure('');
    }
  }

  assertEquals(value1, value2) {
    if (value1 != value2) {
      throw new AssertionFailure('Expected: ' +
        reflectionServices.valToSymbol(value1) + ' but found: ' +
        reflectionServices.valToSymbol(value2));
    }
  }
;

class ExpectationFailure : Exception
  _msg = 'expectation message'

  construct(msg) {
    _msg = msg;
    stack_ = t3GetStackTrace().sublist(2);
    if (stack_.length() > 0 && stack_[1].isSystem())
      stack_ = stack_.sublist(2);
  }

  displayException() {
    "Expectation error: <<_msg>>";
    showStackTrace(stack_);
  }
;

class AssertionFailure : Exception
  _msg = 'assertion message'

  construct(msg) {
    _msg = msg;
    stack_ = t3GetStackTrace().sublist(2);
    if (stack_.length() > 0 && stack_[1].isSystem())
      stack_ = stack_.sublist(2);
  }

  displayException() {
    "Assertion error: <<_msg>>";
    showStackTrace(stack_);
  }
;

class TestRunner : PreinitObject
  total_run = 0
  total_error = 0
  total_failed = 0
  exceptions = []

  execBeforeMe = [reflectionServices]

  execute() {
    for (local klass = firstObj(TestCase, ObjClasses) ; klass != nil ; klass = nextObj(klass, TestCase, ObjClasses)) {
      if (klass.isClass()) {
        foreach (local prop in klass.getPropList()) {
          local instance = klass.createInstance();
          if (reflectionServices.valToSymbol(prop).substr(0, 5) == 'test_') {
            try {
              instance.(prop)();
              say('.');
            } catch (ExpectationFailure expectation) {
              say('F');
              total_failed += 1;
              exceptions = exceptions.append(expectation);
            } catch (AssertionFailure assertion) {
              say('F');
              total_failed += 1;
              exceptions = exceptions.append(assertion);
            } catch (Exception exception) {
              say('E');
              total_error += 1;
              exceptions = exceptions.append(exception);
            }
            total_run += 1;
          }
        }
      }
    }
    say('\n');
    foreach (local exception in exceptions) {
      say('==========================================\n');
      exception.displayException();
    }
    say('==========================================\n');
    say('\n');
    say('' + total_run + ' run  ' + total_error + ' errors '  + total_failed + ' failures\n');
  }
;

testRunner : TestRunner;
